// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_

#include <string.h>
#include <stdint.h>

#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/core/css/parser/css_parser_mode.h"

namespace blink {

enum class CSSValueID {
  kInvalid = 0,
  kInherit = 1,
  kInitial = 2,
  kNone = 3,
  kHidden = 4,
  kInset = 5,
  kGroove = 6,
  kOutset = 7,
  kRidge = 8,
  kDotted = 9,
  kDashed = 10,
  kSolid = 11,
  kDouble = 12,
  kCaption = 13,
  kIcon = 14,
  kMenu = 15,
  kMessageBox = 16,
  kSmallCaption = 17,
  kWebkitMiniControl = 18,
  kWebkitSmallControl = 19,
  kWebkitControl = 20,
  kStatusBar = 21,
  kItalic = 22,
  kOblique = 23,
  kAll = 24,
  kCommonLigatures = 25,
  kNoCommonLigatures = 26,
  kDiscretionaryLigatures = 27,
  kNoDiscretionaryLigatures = 28,
  kHistoricalLigatures = 29,
  kNoHistoricalLigatures = 30,
  kContextual = 31,
  kNoContextual = 32,
  kSmallCaps = 33,
  kAllSmallCaps = 34,
  kPetiteCaps = 35,
  kAllPetiteCaps = 36,
  kUnicase = 37,
  kTitlingCaps = 38,
  kJis78 = 39,
  kJis83 = 40,
  kJis90 = 41,
  kJis04 = 42,
  kSimplified = 43,
  kTraditional = 44,
  kFullWidth = 45,
  kProportionalWidth = 46,
  kRuby = 47,
  kLiningNums = 48,
  kOldstyleNums = 49,
  kProportionalNums = 50,
  kTabularNums = 51,
  kDiagonalFractions = 52,
  kStackedFractions = 53,
  kOrdinal = 54,
  kSlashedZero = 55,
  kStylistic = 56,
  kHistoricalForms = 57,
  kStyleset = 58,
  kCharacterVariant = 59,
  kSwash = 60,
  kOrnaments = 61,
  kAnnotation = 62,
  kNormal = 63,
  kBold = 64,
  kBolder = 65,
  kLighter = 66,
  kWeight = 67,
  kUltraCondensed = 68,
  kExtraCondensed = 69,
  kCondensed = 70,
  kSemiCondensed = 71,
  kSemiExpanded = 72,
  kExpanded = 73,
  kExtraExpanded = 74,
  kUltraExpanded = 75,
  kXxSmall = 76,
  kXSmall = 77,
  kSmall = 78,
  kMedium = 79,
  kLarge = 80,
  kXLarge = 81,
  kXxLarge = 82,
  kXxxLarge = 83,
  kSmaller = 84,
  kLarger = 85,
  kWebkitXxxLarge = 86,
  kExHeight = 87,
  kCapHeight = 88,
  kChWidth = 89,
  kIcWidth = 90,
  kSerif = 91,
  kSansSerif = 92,
  kCursive = 93,
  kFantasy = 94,
  kMonospace = 95,
  kSystemUi = 96,
  kWebkitBody = 97,
  kMath = 98,
  kSwap = 99,
  kFallback = 100,
  kOptional = 101,
  kFontTech = 102,
  kFontFormat = 103,
  kPaletteMix = 104,
  kAqua = 105,
  kBlack = 106,
  kBlue = 107,
  kFuchsia = 108,
  kGray = 109,
  kGreen = 110,
  kLime = 111,
  kMaroon = 112,
  kNavy = 113,
  kOlive = 114,
  kOrange = 115,
  kPurple = 116,
  kRed = 117,
  kSilver = 118,
  kTeal = 119,
  kWhite = 120,
  kYellow = 121,
  kTransparent = 122,
  kWebkitLink = 123,
  kWebkitActivelink = 124,
  kAccentcolor = 125,
  kAccentcolortext = 126,
  kActiveborder = 127,
  kActivecaption = 128,
  kActivetext = 129,
  kAppworkspace = 130,
  kBackground = 131,
  kButtonborder = 132,
  kButtonface = 133,
  kButtonhighlight = 134,
  kButtonshadow = 135,
  kButtontext = 136,
  kCanvas = 137,
  kCanvastext = 138,
  kCaptiontext = 139,
  kField = 140,
  kFieldtext = 141,
  kGraytext = 142,
  kHighlight = 143,
  kHighlighttext = 144,
  kInactiveborder = 145,
  kInactivecaption = 146,
  kInactivecaptiontext = 147,
  kInfobackground = 148,
  kInfotext = 149,
  kLinktext = 150,
  kMark = 151,
  kMarktext = 152,
  kMenutext = 153,
  kSelecteditem = 154,
  kSelecteditemtext = 155,
  kScrollbar = 156,
  kThreeddarkshadow = 157,
  kThreedface = 158,
  kThreedhighlight = 159,
  kThreedlightshadow = 160,
  kThreedshadow = 161,
  kVisitedtext = 162,
  kWindow = 163,
  kWindowframe = 164,
  kWindowtext = 165,
  kInternalActiveListBoxSelection = 166,
  kInternalActiveListBoxSelectionText = 167,
  kInternalInactiveListBoxSelection = 168,
  kInternalInactiveListBoxSelectionText = 169,
  kWebkitFocusRingColor = 170,
  kCurrentcolor = 171,
  kGrey = 172,
  kInternalQuirkInherit = 173,
  kInternalSpellingErrorColor = 174,
  kInternalGrammarErrorColor = 175,
  kAliceblue = 176,
  kAntiquewhite = 177,
  kAquamarine = 178,
  kAzure = 179,
  kBeige = 180,
  kBisque = 181,
  kBlanchedalmond = 182,
  kBlueviolet = 183,
  kBrown = 184,
  kBurlywood = 185,
  kCadetblue = 186,
  kChartreuse = 187,
  kChocolate = 188,
  kCoral = 189,
  kCornflowerblue = 190,
  kCornsilk = 191,
  kCrimson = 192,
  kCyan = 193,
  kDarkblue = 194,
  kDarkcyan = 195,
  kDarkgoldenrod = 196,
  kDarkgray = 197,
  kDarkgreen = 198,
  kDarkgrey = 199,
  kDarkkhaki = 200,
  kDarkmagenta = 201,
  kDarkolivegreen = 202,
  kDarkorange = 203,
  kDarkorchid = 204,
  kDarkred = 205,
  kDarksalmon = 206,
  kDarkseagreen = 207,
  kDarkslateblue = 208,
  kDarkslategray = 209,
  kDarkslategrey = 210,
  kDarkturquoise = 211,
  kDarkviolet = 212,
  kDeeppink = 213,
  kDeepskyblue = 214,
  kDimgray = 215,
  kDimgrey = 216,
  kDodgerblue = 217,
  kFirebrick = 218,
  kFloralwhite = 219,
  kForestgreen = 220,
  kGainsboro = 221,
  kGhostwhite = 222,
  kGold = 223,
  kGoldenrod = 224,
  kGreenyellow = 225,
  kHoneydew = 226,
  kHotpink = 227,
  kIndianred = 228,
  kIndigo = 229,
  kIvory = 230,
  kKhaki = 231,
  kLavender = 232,
  kLavenderblush = 233,
  kLawngreen = 234,
  kLemonchiffon = 235,
  kLightblue = 236,
  kLightcoral = 237,
  kLightcyan = 238,
  kLightgoldenrodyellow = 239,
  kLightgray = 240,
  kLightgreen = 241,
  kLightgrey = 242,
  kLightpink = 243,
  kLightsalmon = 244,
  kLightseagreen = 245,
  kLightskyblue = 246,
  kLightslategray = 247,
  kLightslategrey = 248,
  kLightsteelblue = 249,
  kLightyellow = 250,
  kLimegreen = 251,
  kLinen = 252,
  kMagenta = 253,
  kMediumaquamarine = 254,
  kMediumblue = 255,
  kMediumorchid = 256,
  kMediumpurple = 257,
  kMediumseagreen = 258,
  kMediumslateblue = 259,
  kMediumspringgreen = 260,
  kMediumturquoise = 261,
  kMediumvioletred = 262,
  kMidnightblue = 263,
  kMintcream = 264,
  kMistyrose = 265,
  kMoccasin = 266,
  kNavajowhite = 267,
  kOldlace = 268,
  kOlivedrab = 269,
  kOrangered = 270,
  kOrchid = 271,
  kPalegoldenrod = 272,
  kPalegreen = 273,
  kPaleturquoise = 274,
  kPalevioletred = 275,
  kPapayawhip = 276,
  kPeachpuff = 277,
  kPeru = 278,
  kPink = 279,
  kPlum = 280,
  kPowderblue = 281,
  kRebeccapurple = 282,
  kRosybrown = 283,
  kRoyalblue = 284,
  kSaddlebrown = 285,
  kSalmon = 286,
  kSandybrown = 287,
  kSeagreen = 288,
  kSeashell = 289,
  kSienna = 290,
  kSkyblue = 291,
  kSlateblue = 292,
  kSlategray = 293,
  kSlategrey = 294,
  kSnow = 295,
  kSpringgreen = 296,
  kSteelblue = 297,
  kTan = 298,
  kThistle = 299,
  kTomato = 300,
  kTurquoise = 301,
  kViolet = 302,
  kWheat = 303,
  kWhitesmoke = 304,
  kYellowgreen = 305,
  kRepeat = 306,
  kRepeatX = 307,
  kRepeatY = 308,
  kNoRepeat = 309,
  kClear = 310,
  kCopy = 311,
  kSourceOver = 312,
  kSourceIn = 313,
  kSourceOut = 314,
  kSourceAtop = 315,
  kDestinationOver = 316,
  kDestinationIn = 317,
  kDestinationOut = 318,
  kDestinationAtop = 319,
  kXor = 320,
  kPlusLighter = 321,
  kSubtract = 322,
  kIntersect = 323,
  kExclude = 324,
  kBaseline = 325,
  kMiddle = 326,
  kSub = 327,
  kSuper = 328,
  kTextTop = 329,
  kTextBottom = 330,
  kTop = 331,
  kBottom = 332,
  kWebkitBaselineMiddle = 333,
  kWebkitAuto = 334,
  kLeft = 335,
  kRight = 336,
  kCenter = 337,
  kJustify = 338,
  kWebkitLeft = 339,
  kWebkitRight = 340,
  kWebkitCenter = 341,
  kWebkitMatchParent = 342,
  kInternalCenter = 343,
  kInlineStart = 344,
  kInlineEnd = 345,
  kOutside = 346,
  kInside = 347,
  kDisc = 348,
  kCircle = 349,
  kSquare = 350,
  kDisclosureOpen = 351,
  kDisclosureClosed = 352,
  kDecimal = 353,
  kInline = 354,
  kBlock = 355,
  kFlowRoot = 356,
  kFlow = 357,
  kTable = 358,
  kFlex = 359,
  kGrid = 360,
  kContents = 361,
  kTableRowGroup = 362,
  kTableHeaderGroup = 363,
  kTableFooterGroup = 364,
  kTableRow = 365,
  kTableColumnGroup = 366,
  kTableColumn = 367,
  kTableCell = 368,
  kTableCaption = 369,
  kRubyText = 370,
  kInlineBlock = 371,
  kInlineTable = 372,
  kInlineFlex = 373,
  kInlineGrid = 374,
  kWebkitBox = 375,
  kWebkitInlineBox = 376,
  kWebkitFlex = 377,
  kWebkitInlineFlex = 378,
  kLayout = 379,
  kInlineLayout = 380,
  kListItem = 381,
  kAuto = 382,
  kCrosshair = 383,
  kDefault = 384,
  kPointer = 385,
  kMove = 386,
  kVerticalText = 387,
  kCell = 388,
  kContextMenu = 389,
  kAlias = 390,
  kProgress = 391,
  kNoDrop = 392,
  kNotAllowed = 393,
  kZoomIn = 394,
  kZoomOut = 395,
  kEResize = 396,
  kNeResize = 397,
  kNwResize = 398,
  kNResize = 399,
  kSeResize = 400,
  kSwResize = 401,
  kSResize = 402,
  kWResize = 403,
  kEwResize = 404,
  kNsResize = 405,
  kNeswResize = 406,
  kNwseResize = 407,
  kColResize = 408,
  kRowResize = 409,
  kText = 410,
  kWait = 411,
  kHelp = 412,
  kAllScroll = 413,
  kGrab = 414,
  kGrabbing = 415,
  kWebkitGrab = 416,
  kWebkitGrabbing = 417,
  kWebkitZoomIn = 418,
  kWebkitZoomOut = 419,
  kLtr = 420,
  kRtl = 421,
  kCapitalize = 422,
  kUppercase = 423,
  kLowercase = 424,
  kMathAuto = 425,
  kVisible = 426,
  kCollapse = 427,
  kPreserve = 428,
  kPreserveBreaks = 429,
  kPretty = 430,
  kA3 = 431,
  kA4 = 432,
  kA5 = 433,
  kAbove = 434,
  kAbsolute = 435,
  kAlways = 436,
  kAvoid = 437,
  kB4 = 438,
  kB5 = 439,
  kBelow = 440,
  kBidiOverride = 441,
  kBlink = 442,
  kBoth = 443,
  kBreakSpaces = 444,
  kCloseQuote = 445,
  kEmbed = 446,
  kFixed = 447,
  kHand = 448,
  kHide = 449,
  kIsolate = 450,
  kIsolateOverride = 451,
  kPlaintext = 452,
  kWebkitIsolate = 453,
  kWebkitIsolateOverride = 454,
  kWebkitPlaintext = 455,
  kJisB5 = 456,
  kJisB4 = 457,
  kLandscape = 458,
  kLedger = 459,
  kLegal = 460,
  kLetter = 461,
  kLineThrough = 462,
  kLocal = 463,
  kNoCloseQuote = 464,
  kNoOpenQuote = 465,
  kNowrap = 466,
  kOpenQuote = 467,
  kOverlay = 468,
  kOverline = 469,
  kPortrait = 470,
  kPre = 471,
  kPreLine = 472,
  kPreWrap = 473,
  kRelative = 474,
  kScroll = 475,
  kSeparate = 476,
  kShow = 477,
  kStatic = 478,
  kThick = 479,
  kThin = 480,
  kUnderline = 481,
  kView = 482,
  kWavy = 483,
  kCompact = 484,
  kStretch = 485,
  kStart = 486,
  kEnd = 487,
  kClone = 488,
  kSlice = 489,
  kReverse = 490,
  kHorizontal = 491,
  kVertical = 492,
  kInlineAxis = 493,
  kBlockAxis = 494,
  kFlexStart = 495,
  kFlexEnd = 496,
  kSpaceBetween = 497,
  kSpaceAround = 498,
  kSpaceEvenly = 499,
  kUnsafe = 500,
  kSafe = 501,
  kAnchorCenter = 502,
  kRow = 503,
  kRowReverse = 504,
  kColumn = 505,
  kColumnReverse = 506,
  kWrap = 507,
  kWrapReverse = 508,
  kAutoFlow = 509,
  kDense = 510,
  kReadOnly = 511,
  kReadWrite = 512,
  kReadWritePlaintextOnly = 513,
  kElement = 514,
  kWebkitMinContent = 515,
  kWebkitMaxContent = 516,
  kWebkitFillAvailable = 517,
  kWebkitFitContent = 518,
  kMinContent = 519,
  kMaxContent = 520,
  kFitContent = 521,
  kNoAutospace = 522,
  kClip = 523,
  kEllipsis = 524,
  kSpellingError = 525,
  kGrammarError = 526,
  kFromFont = 527,
  kSpaceAll = 528,
  kSpaceFirst = 529,
  kTrimStart = 530,
  kBreakAll = 531,
  kKeepAll = 532,
  kAutoPhrase = 533,
  kBreakWord = 534,
  kSpace = 535,
  kLoose = 536,
  kStrict = 537,
  kAfterWhiteSpace = 538,
  kAnywhere = 539,
  kManual = 540,
  kCheckbox = 541,
  kRadio = 542,
  kButton = 543,
  kListbox = 544,
  kInternalMediaControl = 545,
  kMenulist = 546,
  kMenulistButton = 547,
  kMeter = 548,
  kProgressBar = 549,
  kSearchfield = 550,
  kTextfield = 551,
  kTextarea = 552,
  kInnerSpinButton = 553,
  kPushButton = 554,
  kSquareButton = 555,
  kSliderHorizontal = 556,
  kSearchfieldCancelButton = 557,
  kMediaSlider = 558,
  kMediaSliderthumb = 559,
  kMediaVolumeSlider = 560,
  kMediaVolumeSliderthumb = 561,
  kSliderthumbHorizontal = 562,
  kSliderthumbVertical = 563,
  kSliderVertical = 564,
  kRound = 565,
  kBorder = 566,
  kBorderBox = 567,
  kContent = 568,
  kContentBox = 569,
  kPadding = 570,
  kPaddingBox = 571,
  kMarginBox = 572,
  kNoClip = 573,
  kContain = 574,
  kCover = 575,
  kLogical = 576,
  kVisual = 577,
  kReplace = 578,
  kAccumulate = 579,
  kAlternate = 580,
  kAlternateReverse = 581,
  kForwards = 582,
  kBackwards = 583,
  kInfinite = 584,
  kRunning = 585,
  kPaused = 586,
  kFlat = 587,
  kPreserve3d = 588,
  kFillBox = 589,
  kViewBox = 590,
  kEase = 591,
  kLinear = 592,
  kEaseIn = 593,
  kEaseOut = 594,
  kEaseInOut = 595,
  kJumpBoth = 596,
  kJumpEnd = 597,
  kJumpNone = 598,
  kJumpStart = 599,
  kStepStart = 600,
  kStepEnd = 601,
  kSteps = 602,
  kFrames = 603,
  kCubicBezier = 604,
  kDocument = 605,
  kReset = 606,
  kZoom = 607,
  kVisiblepainted = 608,
  kVisiblefill = 609,
  kVisiblestroke = 610,
  kPainted = 611,
  kFill = 612,
  kStroke = 613,
  kBoundingBox = 614,
  kSpellOut = 615,
  kDigits = 616,
  kLiteralPunctuation = 617,
  kNoPunctuation = 618,
  kAntialiased = 619,
  kSubpixelAntialiased = 620,
  kOptimizespeed = 621,
  kOptimizelegibility = 622,
  kGeometricprecision = 623,
  kCrispedges = 624,
  kEconomy = 625,
  kExact = 626,
  kLr = 627,
  kRl = 628,
  kTb = 629,
  kLrTb = 630,
  kRlTb = 631,
  kTbRl = 632,
  kHorizontalTb = 633,
  kVerticalRl = 634,
  kVerticalLr = 635,
  kAfter = 636,
  kBefore = 637,
  kOver = 638,
  kUnder = 639,
  kFilled = 640,
  kOpen = 641,
  kDot = 642,
  kDoubleCircle = 643,
  kTriangle = 644,
  kSesame = 645,
  kEllipse = 646,
  kClosestSide = 647,
  kClosestCorner = 648,
  kFarthestSide = 649,
  kFarthestCorner = 650,
  kMixed = 651,
  kSideways = 652,
  kSidewaysRight = 653,
  kUpright = 654,
  kVerticalRight = 655,
  kOn = 656,
  kOff = 657,
  kOptimizequality = 658,
  kPixelated = 659,
  kWebkitOptimizeContrast = 660,
  kFromImage = 661,
  kRotateLeft = 662,
  kRotateRight = 663,
  kNonzero = 664,
  kEvenodd = 665,
  kAt = 666,
  kAlphabetic = 667,
  kBorderless = 668,
  kFullscreen = 669,
  kStandalone = 670,
  kMinimalUi = 671,
  kBrowser = 672,
  kWindowControlsOverlay = 673,
  kTabbed = 674,
  kMinimized = 675,
  kMaximized = 676,
  kPaged = 677,
  kSlow = 678,
  kFast = 679,
  kSticky = 680,
  kCoarse = 681,
  kFine = 682,
  kOnDemand = 683,
  kHover = 684,
  kMultiply = 685,
  kScreen = 686,
  kDarken = 687,
  kLighten = 688,
  kColorDodge = 689,
  kColorBurn = 690,
  kHardLight = 691,
  kSoftLight = 692,
  kDifference = 693,
  kExclusion = 694,
  kHue = 695,
  kSaturation = 696,
  kColor = 697,
  kLuminosity = 698,
  kScaleDown = 699,
  kBalance = 700,
  kDrag = 701,
  kNoDrag = 702,
  kSpan = 703,
  kMinmax = 704,
  kSubgrid = 705,
  kProgressive = 706,
  kInterlace = 707,
  kMarkers = 708,
  kAlpha = 709,
  kLuminance = 710,
  kMatchSource = 711,
  kSRGB = 712,
  kLinearrgb = 713,
  kButt = 714,
  kMiter = 715,
  kBevel = 716,
  kBeforeEdge = 717,
  kAfterEdge = 718,
  kCentral = 719,
  kTextBeforeEdge = 720,
  kTextAfterEdge = 721,
  kIdeographic = 722,
  kHanging = 723,
  kMathematical = 724,
  kUseScript = 725,
  kNoChange = 726,
  kResetSize = 727,
  kDynamic = 728,
  kNonScalingStroke = 729,
  kInternalExtendToZoom = 730,
  kPanX = 731,
  kPanY = 732,
  kPanLeft = 733,
  kPanRight = 734,
  kPanUp = 735,
  kPanDown = 736,
  kManipulation = 737,
  kPinchZoom = 738,
  kLastBaseline = 739,
  kFirstBaseline = 740,
  kFirst = 741,
  kLast = 742,
  kSelfStart = 743,
  kSelfEnd = 744,
  kLegacy = 745,
  kSmooth = 746,
  kScrollPosition = 747,
  kRevert = 748,
  kRevertLayer = 749,
  kUnset = 750,
  kLinearGradient = 751,
  kRadialGradient = 752,
  kConicGradient = 753,
  kRepeatingLinearGradient = 754,
  kRepeatingRadialGradient = 755,
  kRepeatingConicGradient = 756,
  kPaint = 757,
  kCrossFade = 758,
  kWebkitCrossFade = 759,
  kWebkitGradient = 760,
  kWebkitLinearGradient = 761,
  kWebkitRadialGradient = 762,
  kWebkitRepeatingLinearGradient = 763,
  kWebkitRepeatingRadialGradient = 764,
  kWebkitImageSet = 765,
  kImageSet = 766,
  kType = 767,
  kTo = 768,
  kColorStop = 769,
  kRadial = 770,
  kAttr = 771,
  kCounter = 772,
  kCounters = 773,
  kRect = 774,
  kPolygon = 775,
  kFormat = 776,
  kCollection = 777,
  kEmbeddedOpentype = 778,
  kOpentype = 779,
  kSVG = 780,
  kTruetype = 781,
  kWoff = 782,
  kWoff2 = 783,
  kTech = 784,
  kFeaturesOpentype = 785,
  kFeaturesAat = 786,
  kFeaturesGraphite = 787,
  kColorCOLRv0 = 788,
  kColorCOLRv1 = 789,
  kColorSVG = 790,
  kColorSbix = 791,
  kColorCBDT = 792,
  kVariations = 793,
  kPalettes = 794,
  kIncremental = 795,
  kInvert = 796,
  kGrayscale = 797,
  kSepia = 798,
  kSaturate = 799,
  kHueRotate = 800,
  kOpacity = 801,
  kBrightness = 802,
  kContrast = 803,
  kBlur = 804,
  kDropShadow = 805,
  kUrl = 806,
  kRgb = 807,
  kRgba = 808,
  kHsl = 809,
  kHsla = 810,
  kHwb = 811,
  kLab = 812,
  kOklab = 813,
  kLch = 814,
  kOklch = 815,
  kInternalLightDark = 816,
  kSRGBLinear = 817,
  kDisplayP3 = 818,
  kA98Rgb = 819,
  kProphotoRgb = 820,
  kXyz = 821,
  kXyzD50 = 822,
  kXyzD65 = 823,
  kShorter = 824,
  kLonger = 825,
  kDecreasing = 826,
  kIncreasing = 827,
  kIn = 828,
  kColorMix = 829,
  kFrom = 830,
  kR = 831,
  kG = 832,
  kB = 833,
  kH = 834,
  kS = 835,
  kL = 836,
  kW = 837,
  kA = 838,
  kC = 839,
  kMatrix = 840,
  kMatrix3d = 841,
  kPerspective = 842,
  kRotate = 843,
  kRotateX = 844,
  kRotateY = 845,
  kRotateZ = 846,
  kRotate3d = 847,
  kScale = 848,
  kScaleX = 849,
  kScaleY = 850,
  kScaleZ = 851,
  kScale3d = 852,
  kSkew = 853,
  kSkewX = 854,
  kSkewY = 855,
  kTranslate = 856,
  kTranslateX = 857,
  kTranslateY = 858,
  kTranslateZ = 859,
  kTranslate3d = 860,
  kX = 861,
  kY = 862,
  kZ = 863,
  kPath = 864,
  kRay = 865,
  kSides = 866,
  kStrokeBox = 867,
  kCalc = 868,
  kWebkitCalc = 869,
  kMin = 870,
  kMax = 871,
  kClamp = 872,
  kSin = 873,
  kCos = 874,
  kAsin = 875,
  kAtan = 876,
  kAtan2 = 877,
  kAcos = 878,
  kMod = 879,
  kRem = 880,
  kUp = 881,
  kDown = 882,
  kToZero = 883,
  kSign = 884,
  kAbs = 885,
  kPow = 886,
  kSqrt = 887,
  kHypot = 888,
  kLog = 889,
  kExp = 890,
  kInfinity = 891,
  kNegativeInfinity = 892,
  kNan = 893,
  kPi = 894,
  kE = 895,
  kMandatory = 896,
  kProximity = 897,
  kStyle = 898,
  kSize = 899,
  kBlockSize = 900,
  kInlineSize = 901,
  kScrollState = 902,
  kInsetBlockStart = 903,
  kInsetBlockEnd = 904,
  kInsetInlineStart = 905,
  kInsetInlineEnd = 906,
  kAutoFill = 907,
  kAutoFit = 908,
  kVar = 909,
  kInternalVariableValue = 910,
  kEnv = 911,
  kAvoidPage = 912,
  kPage = 913,
  kRecto = 914,
  kVerso = 915,
  kAvoidColumn = 916,
  kP3 = 917,
  kRec2020 = 918,
  kAdd = 919,
  kAutoAdd = 920,
  kTrue = 921,
  kFalse = 922,
  kNoPreference = 923,
  kDark = 924,
  kLight = 925,
  kOnly = 926,
  kReduce = 927,
  kActive = 928,
  kPreserveParentColor = 929,
  kBackButton = 930,
  kFabricated = 931,
  kSelector = 932,
  kContinuous = 933,
  kFolded = 934,
  kStable = 935,
  kBothEdges = 936,
  kMore = 937,
  kLess = 938,
  kCustom = 939,
  kCyclic = 940,
  kSymbolic = 941,
  kNumeric = 942,
  kAdditive = 943,
  kExtends = 944,
  kInternalSimpChineseInformal = 945,
  kInternalSimpChineseFormal = 946,
  kInternalTradChineseInformal = 947,
  kInternalTradChineseFormal = 948,
  kInternalKoreanHangulFormal = 949,
  kInternalKoreanHanjaInformal = 950,
  kInternalKoreanHanjaFormal = 951,
  kInternalHebrew = 952,
  kInternalLowerArmenian = 953,
  kInternalUpperArmenian = 954,
  kInternalEthiopicNumeric = 955,
  kBullets = 956,
  kNumbers = 957,
  kWords = 958,
  kStandard = 959,
  kHigh = 960,
  kConstrainedHigh = 961,
  kDynamicRangeLimitMix = 962,
  kLayer = 963,
  kSupports = 964,
  kColorContrast = 965,
  kVs = 966,
  kAA = 967,
  kAALarge = 968,
  kAAA = 969,
  kAAALarge = 970,
  kDrop = 971,
  kRaise = 972,
  kXywh = 973,
  kAnchor = 974,
  kAutoSame = 975,
  kAnchorSize = 976,
  kWidth = 977,
  kHeight = 978,
  kSelfBlock = 979,
  kSelfInline = 980,
  kImplicit = 981,
  kEntry = 982,
  kExit = 983,
  kEntryCrossing = 984,
  kExitCrossing = 985,
  kRoot = 986,
  kNearest = 987,
  kSelf = 988,
  kAllowDiscrete = 989,
  kInverted = 990,
  kEnabled = 991,
  kInitialOnly = 992,
  kXStart = 993,
  kXEnd = 994,
  kYStart = 995,
  kYEnd = 996,
  kXSelfStart = 997,
  kXSelfEnd = 998,
  kYSelfStart = 999,
  kYSelfEnd = 1000,
  kInternalTextareaAuto = 1001,
};

const int numCSSValueKeywords = 1002;
const size_t maxCSSValueKeywordLength = 42;

inline bool IsValidCSSValueID(CSSValueID id)
{
    return id != CSSValueID::kInvalid;
}

CORE_EXPORT const char* getValueName(CSSValueID);
bool isValueAllowedInMode(CSSValueID id, CSSParserMode mode);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
